
struct DerivedGeometryWrapper
    geometry_expr::Expr
    parent_key::UInt
end

mutable struct EquilibriumWrapper{T, E <: AbstractMagneticEquilibrium}
    eq::E
    input::OrderedDict
    translator::Dict{Int, Symbol}
    derived_geometries::Dict{UInt, Expr}
    EquilibriumWrapper{T, E}() where {T, E <: AbstractMagneticEquilibrium} = new{T, E}(NullEquilibrium(), OrderedDict{Symbol, Any}(), Dict{Int, Symbol}(), Dict{UInt, Expr}())
    EquilibriumWrapper{T, E}(a, b, c, d) where {T, E <: AbstractMagneticEquilibrium} = new{T, E}(a, b, c, d)
end
  
function EquilibriumWrapper(::E) where {E <: AbstractMagneticEquilibrium}
    error("EquilibriumWrapper constructor for $(E) not implemented")
end

function EquilibriumWrapper{T, E}(::AbstractDict) where {T, E <: AbstractMagneticEquilibrium}
    error("EquilibriumWrapper constructor for $(E) from an input dictionary not yet implemented")
end

function EquilibriumWrapper{T, E}(::AbstractString) where {T, E <: AbstractMagneticEquilibrium}
    error("EquilibriumWrapper constructor for $(E) from an input file not yet implemented")
end

function equilibrium_type(::EquilibriumWrapper{T, E}) where {T, E <: AbstractMagneticEquilibrium}
    return E
end

function set_input!(wrapped_eq::EquilibriumWrapper{T, E},
                    new_input::OrderedDict;
                   ) where {T, E <: AbstractMagneticEquilibrium}
    for (key, value) in new_input
        wrapped_eq.input[key] = value
    end
end

function update_input!(::EquilibriumWrapper{T, E}) where {T, E <: AbstractMagneticEquilibrium}
    error("Method for updating wrapper input from wrapper equilibrium for EquilibriumWrapper{$E} not yet implemented")
end

function update_input!(wrapped_eq::EquilibriumWrapper{T, E},
                       index::Int,
                       entry;
                     ) where {T, E <: AbstractMagneticEquilibrium}
    wrapped_eq.input[wrapped_eq.translator[index]] = entry
    nothing
end

function set_translator!(wrapped_eq::EquilibriumWrapper{T, E},
                         new_translator::AbstractDict{Int, Symbol};
                        ) where {T, E <: AbstractMagneticEquilibrium}
    for (key, value) in new_translator
        wrapped_eq.translator[key] = value
    end
    nothing
end

function update_translator!(wrapped_eq::EquilibriumWrapper{T, E},
                            index::Int,
                            key::Symbol;
                           ) where {T, E <: AbstractMagneticEquilibrium}
    wrapped_eq.translator[index] = key
    nothing
end


function translate!(::EquilibriumWrapper{T, E},
                    ::Vector{V};
                   ) where {T, E <: AbstractMagneticEquilibrium, V <: AbstractOptVariable}
    error("Method for translating state vector into equilibrium input for EquilibriumWrapper{$(E)} not implemented")
end

function compute_equilibrium(::EquilibriumWrapper{T, E} where {T, E <: AbstractMagneticEquilibrium})
    error("Method for computing equilbrium for EquilibriumWrapper{$(E)} not implemented")
end

function update_equilibrium!(wrapped_eq::EquilibriumWrapper{T, E},
                             new_eq::E;
                            ) where {T, E <: AbstractMagneticEquilibrium}
    setfield!(wrapped_eq, :eq, new_eq)
    nothing
end

function add_derived_geometry!(wrapped_eq::EquilibriumWrapper{T, E},
                               key::UInt,
                               geometry_expr::Expr;
                              ) where {T, E <: AbstractMagneticEquilibrium}
    wrapped_eq.derived_geometries[key] = geometry_expr
    return nothing
end

function compute_target(::EquilibriumWrapper{T, E},
                        ::Expr;
                       ) where {T, E <: AbstractMagneticEquilibrium}
    error("`compute_target` function for EquilibriumWrapper{$(E)} not implemented")
end

function extract_derived_geometry(::EquilibriumWrapper{T, E},
                                  key::UInt;
                                 ) where {T, E <: AbstractMagneticEquilibrium}
    error("extract_derived_geometry function not implemented for EquilibriumWrapper{$E}")
end

function write_input_file(::EquilibriumWrapper{T, E},
                          filename::AbstractString;
                         ) where {T, E <: AbstractMagneticEquilibrium}
    error("write_input_file not implemented for EquilibriumWrapper{$E}")
end
