using Optim, PlasmaEquilibriumToolkit, NE3DLE, StellaratorOptimization, LineSearches
#ENV["JULIA_DEBUG"] = StellOpt

optimizer = StellOpt.Optimizer(ConjugateGradient(; alphaguess=1e-5, linesearch=BackTracking()),
                               Optim.Options(iterations = 5))
prob = StellOpt.Problem()
StellOpt.add_optimizer!(prob, optimizer)

nml = NE3DLE.read_par_file("/home/bfaber/.julia/dev/NE3DLE/NE3DLE.in")
Ne3dle_wrapper = StellOpt.EquilibriumWrapper{Float64, NE3DLESurface}(nml)
eq = call_NE3DLE(nml)
StellOpt.update_equilibrium!(Ne3dle_wrapper, eq)
StellOpt.add_equilibrium_wrapper!(prob, Ne3dle_wrapper)
Ne3dle_s = StellOpt.@add_derived_geometry!(prob,NE3DLESurface(equilibrium=Ne3dle_wrapper))
StellOpt.@add_target!(prob, NE3DLE.quasisymmetry_deviation(geometry=Ne3dle_s), 0.0, 1.0, "qs_ne3dle")
StellOpt.@add_target!(prob, NE3DLE.magnetic_well(geometry = Ne3dle_s), -0.1, 1.0, "well_ne3dle")
#=
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(0,1)", fd_step = 0.01)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(0,-1)", fd_step = 0.01)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(1,0)", fd_step = 0.01)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(0,1)", fd_step = 0.01)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(0,-1)", fd_step = 0.01)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(1,0)", fd_step = 0.01)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(0,2)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(0,-2)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(2,0)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(-2,0)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(0,2)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(0,-2)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(2,0)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(-2,0)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(1,2)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(1,-2)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(2,1)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(-2,1)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(1,2)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(1,-2)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(2,1)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(-2,1)", fd_step = 0.001)
Opt.add_variable!(prob, Ne3dle_wrapper, "Rmn(-1,2)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(-1,-2)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(2,-1)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(-2,-1)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(-1,2)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(-1,-2)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(2,-1)", fd_step = 0.001)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(-2,-1)", fd_step = 0.001)
=#
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(0,1)", fd_step = 0.01)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(1,0)", fd_step = 0.01)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(0,1)", fd_step = 0.01)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(1,0)", fd_step = 0.01)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(1,2)", fd_step = 1e-4)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Rmn(2,1)", fd_step = 1e-4)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(1,2)", fd_step = 1e-4)
StellOpt.add_variable!(prob, Ne3dle_wrapper, "Zmn(2,1)", fd_step = 1e-4)
#residual = StellOpt.optimize_threads(prob)